@@ -1,6 +1,6 @@
 /* Target-dependent code for the HP PA-RISC architecture.
 
-   Copyright (C) 2003-2022 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,14 +17,14 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef HPPA_TDEP_H
-#define HPPA_TDEP_H
+#ifndef GDB_HPPA_TDEP_H
+#define GDB_HPPA_TDEP_H
 
 #include "gdbarch.h"
 
 struct trad_frame_saved_reg;
 struct objfile;
-struct so_list;
+struct solib;
 
 /* Register numbers of various important registers.  */
 
@@ -109,12 +109,12 @@ struct hppa_gdbarch_tdep : gdbarch_tdep_base
      not interested in them.  If we detect that we are returning to a stub,
      adjust the pc to the real caller.  This improves the behavior of commands
      that traverse frames such as "up" and "finish".  */
-  void (*unwind_adjust_stub) (frame_info_ptr this_frame, CORE_ADDR base,
+  void (*unwind_adjust_stub) (const frame_info_ptr &this_frame, CORE_ADDR base,
 			      trad_frame_saved_reg *saved_regs) = nullptr;
 
   /* These are solib-dependent methods.  They are really HPUX only, but
      we don't have a HPUX-specific tdep vector at the moment.  */
-  CORE_ADDR (*solib_thread_start_addr) (struct so_list *so) = nullptr;
+  CORE_ADDR (*solib_thread_start_addr) (solib *so) = nullptr;
   CORE_ADDR (*solib_get_got_by_pc) (CORE_ADDR addr) = nullptr;
   CORE_ADDR (*solib_get_solib_by_pc) (CORE_ADDR addr) = nullptr;
   CORE_ADDR (*solib_get_text_base) (struct objfile *objfile) = nullptr;
@@ -201,17 +201,17 @@ int hppa_extract_14 (unsigned);
 CORE_ADDR hppa_symbol_address(const char *sym);
 
 extern struct value *
-  hppa_frame_prev_register_helper (frame_info_ptr this_frame,
+  hppa_frame_prev_register_helper (const frame_info_ptr &this_frame,
 				   trad_frame_saved_reg *saved_regs,
 				   int regnum);
 
 extern CORE_ADDR hppa_read_pc (struct regcache *regcache);
 extern void hppa_write_pc (struct regcache *regcache, CORE_ADDR pc);
 extern CORE_ADDR hppa_unwind_pc (struct gdbarch *gdbarch,
-				 frame_info_ptr next_frame);
+				 const frame_info_ptr &next_frame);
 
 extern int hppa_in_solib_call_trampoline (struct gdbarch *gdbarch,
 					  CORE_ADDR pc);
-extern CORE_ADDR hppa_skip_trampoline_code (frame_info_ptr, CORE_ADDR pc);
+extern CORE_ADDR hppa_skip_trampoline_code (const frame_info_ptr &, CORE_ADDR pc);
 
-#endif  /* hppa-tdep.h */
+#endif /* GDB_HPPA_TDEP_H */
